@@ -2,9 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2022 */
+/* Copyright (c) The Exim Maintainers 2020 - 2023 */
 /* Copyright (c) University of Cambridge 1995 - 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* This file is the header that is the only Exim header to be included in the
 source for the local_scan.c() function. It contains definitions that are made
@@ -108,12 +109,22 @@ enum { opt_stringptr, opt_int, opt_octint, opt_mkint, opt_Kint, opt_fixed,
 by exim. The external version for use in Received: strings has a leading 'E'
 added to ensure it starts with a letter. */
 
-#define MESSAGE_ID_LENGTH 16
+#define MESSAGE_ID_PID_LEN_OLD		6
+#define MESSAGE_ID_SUBTIME_LEN_OLD	2
+
+/* tttttt-ppppppppppp-ssss */
+# define MESSAGE_ID_TIME_LEN	6	/*III could these be not-exposed to local_scan? */
+# define MESSAGE_ID_PID_LEN	11
+# define MESSAGE_ID_SUBTIME_LEN	4
+
+#define MESSAGE_ID_LENGTH_OLD	(MESSAGE_ID_TIME_LEN+1+MESSAGE_ID_PID_LEN_OLD+1+MESSAGE_ID_SUBTIME_LEN_OLD)
+#define MESSAGE_ID_LENGTH	(MESSAGE_ID_TIME_LEN+1+MESSAGE_ID_PID_LEN    +1+MESSAGE_ID_SUBTIME_LEN)
 
 /* The offset to the start of the data in the data file - this allows for
 the name of the data file to be present in the first line. */
 
-#define SPOOL_DATA_START_OFFSET (MESSAGE_ID_LENGTH+3)
+#define SPOOL_DATA_START_OFFSET_OLD	(MESSAGE_ID_LENGTH_OLD+3)
+#define SPOOL_DATA_START_OFFSET		(MESSAGE_ID_LENGTH+3)
 
 /* Structure definitions that are documented as visible in the function. */
 
@@ -132,7 +143,7 @@ typedef struct {
   union {
     void *	value;
     long	offset;
-    void (*	fn)();
+    void (*	fn)(const uschar *, const uschar *, unsigned);
   } v;
 } optionlist;
 #define OPT_OFF(s, field) {.offset = offsetof(s, field)}
@@ -187,8 +198,8 @@ extern uschar *expand_string(uschar *);
 extern void    header_add(int, const char *, ...);
 extern void    header_add_at_position(BOOL, uschar *, BOOL, int, const char *, ...);
 extern void    header_remove(int, const uschar *);
-extern BOOL    header_testname(header_line *, const uschar *, int, BOOL);
-extern BOOL    header_testname_incomplete(header_line *, const uschar *, int, BOOL);
+extern BOOL    header_testname(const header_line *, const uschar *, int, BOOL);
+extern BOOL    header_testname_incomplete(const header_line *, const uschar *, int, BOOL);
 extern void    log_write(unsigned int, int, const char *format, ...) PRINTF_FUNCTION(3,4);
 extern int     lss_b64decode(uschar *, uschar **);
 extern uschar *lss_b64encode(uschar *, int);
